export const commandIDs = {
  openDocManager: "docmanager:open",
  newDocManager: "docmanager:new-untitled",
  saveDocManager: "docmanager:save",
  reloadDocManager: "docmanager:reload",
  createNewXircuit: "Xircuit-editor:create-new",
  copyXircuitsToRoot: "Xircuit-editor:copy-to-root",
  saveXircuit: "Xircuit-editor:save-node",
  compileXircuit: "Xircuit-editor:compile-node",
  runXircuit: "Xircuit-editor:run-node",
  lockXircuit: "Xircuit-editor:lock-node",
  openScript: "Xircuit-editor:open-node-script",
  openXircuitsWorkflow: "Xircuit-editor:open-xircuits-workflow",
  undo: "Xircuit-editor:undo",
  redo: "Xircuit-editor:redo",
  cutNode: "Xircuit-editor:cut-node",
  copyNode: "Xircuit-editor:copy-node",
  pasteNode: "Xircuit-editor:paste-node",
  attachNode: "Xircuit-editor:attach-node",
  attachAllNodes: "Xircuit-editor:attach-all-nodes",
  detachAllNodes: "Xircuit-editor:detach-all-nodes",
  triggerLoadingAnimation: "Xircuit-editor:trigger-loading-animation",
  reloadNode: "Xircuit-editor:reload-node",
  reloadAllNodes: "Xircuit-editor:reload-all-nodes",
  toggleAllLinkAnimation: "Xircuit-editor:toggle-all-link-animation",
  editNode: "Xircuit-editor:edit-node",
  deleteEntity: "Xircuit-editor:delete-entities",
  addNodeGivenPosition: "Xircuit-editor:add-node",
  connectNodeByLink: "Xircuit-editor:connect-node",
  connectLinkToObviousPorts: "Xircuit-editor:connect-obvious-link",
  addCommentNode: "Xircuit-editor:add-comment-node",
  compileFile: "Xircuit-editor:compile-file",
  nextNode: "Xircuit-editor:next-node",
  outputMsg: "Xircuit-log:logOutputMessage",
  executeToOutputPanel: "Xircuit-output-panel:execute",
  createNewComponentLibrary: "Xircuit-editor:new-component-library",
  refreshComponentList: "xircuits-sidebar:refresh-component-list",
  toggleDisplayNodesInLibrary: "xircuits-sidebar:toggle-display-nodes-in-library",
  helpOpenResource: "xircuits-help:open-resource",
  openXircuitsConfiguration: "xircuits-config:open-config",
  fetchRemoteRunConfig: "xircuits-config:fetch-remote-config",
  openTvbExtUnicore: 'tvb_ext_unicore:open',
  openViewer: 'Xircuit-editor:open-node-viewer'
};