
 /* -------------------------------------------------- */
  /* ---------------- Accordion ---------------------- */
  /* -------------------------------------------------- */

  .accordion {
    border: 0px solid rgba(0, 0, 0, 0.1);
    /* border: 1px solid rgba(0, 0, 0, 0.1);
    border-radius: 3px; */
  }
  
  .accordion__item + .accordion__item {
    margin-top: 7px;
  }
  
  .accordion__button {
    cursor: pointer;
    padding: 10px;
    width: auto;
    text-align: left;
    border-bottom: 1px solid var(--jp-border-color0);
    font-weight: 600;
    letter-spacing: 0.025em;

    color: var(--jp-ui-font-color0);
    background-color: var(--jp-layout-color0);

    display: flex;
    align-items: center;
  }

  .accordion__button[aria-expanded="true"],
  .accordion__button[aria-selected="true"] {
    box-shadow: 0px 5px 5px 0px oklch(0 0 0 / 0.3);
  }

  .accordion__button[aria-expanded="true"] > a.button,
  .accordion__button[aria-selected="true"] > a.button {
    display: block;
  }


  .accordion__button:hover {
    background: oklch(0 0 0 / 0.05);
  }
  
  .accordion__button:before {
    display: inline-block;
    content: "";
    height: 5px;
    width: 5px;
    margin-right: 12px;
    border-bottom: 2px solid currentColor;
    border-right: 2px solid currentColor;
    transform: rotate(-45deg);
  }
  
  .accordion__button[aria-expanded="true"]::before,
  .accordion__button[aria-selected="true"]::before {
    transform: rotate(45deg);
  }
  
  [hidden] {
    display: none;
  }
  
  .accordion__panel {
    padding: 10px 15px;
    animation: fadein 0.45s ease-in;
    border-bottom: 1px solid oklch(0 0 0 / 0.5);
  }
  
  .accordion__button--remote {
    cursor: default;
  }

  .accordion__button--remote > a.button {
    display: block !important;
  }

  .accordion__button--remote:before {
    opacity: 0;
  }

  .accordion__button--remote:hover {
    background: transparent;
  }

  .accordion__button--remote[aria-expanded="true"],
  .accordion__button--remote[aria-selected="true"]{
    box-shadow: 0 0 0;
  }

  .accordion__button > span {
    flex-grow: 1;
  }

  .accordion__button > a.button {
    padding: 7px 7px;
    margin: -7px 0;
    height: 100%;
    display: none;
    cursor: pointer;
  }

  .accordion__button > a.button:hover {
    background: oklch(0 0 0 / 0.05);
  }

  /* -------------------------------------------------- */
  /* ---------------- Animation part ------------------ */
  /* -------------------------------------------------- */
  
  @keyframes fadein {
    0% {
      opacity: 0;
    }
  
    100% {
      opacity: 1;
    }
  }
  
 /* -------------------------------------------------- */
  /* ----------------Input text ---------------------- */
  /* -------------------------------------------------- */

  a {
    text-decoration: none;
  }
  .search-input {
    top: 50%;
    left: 50%;
    height: 20px;
    border-radius: 7px;
    padding: 10px;
    background: oklch(0.781 0.112 276.768 / 0.2);
    flex-grow: 1;
  }
  .search-input__button {
    border-radius: 7px;
    color: oklch(0.781 0.112 276.768 / 0.75);
    float: left;
    width: 40px;
    height: 40px;
    justify-content: center;
    align-items: center;
  }
  .search-input__text-input {
    border: none;
    background: none;
    outline: none;
    float: left;
    padding: 0;
    color: #000000;
    font-size: 14px;
    line-height: 40px;
    width: 0px;
  }
  ::placeholder {
    padding-left: 15px;
    color: #000000;
  }
  .search-input > .search-input__text-input {
    width: 150px;
  }
  
  .search-input__text-input {
    line-height: 20px;
  }
  .search-input__button {
    width: 20px;
    height: 20px;
  }

 .sidebar-header {
   display: flex;
   width: 100%;
   gap: 0.25em;
   align-items: stretch;
   margin: 10px 10px 15px;
 }

 .sidebar-header > a.button {
   cursor: pointer;
   padding: 3px;
   display: flex;
   align-items: center;
   min-width: 20px;
   margin-right: 5px;
 }

 .sidebar-header > a.button:hover {
   background-color: oklch(0 0 0 / 0.1);
 }

 #sidebar-tooltip.show {
   opacity: 1;
   box-shadow: 1px 1px 10px 0px rgb(0 0 0 / 0.5);
 }
 .sidebar-context-menu .lm-Menu-itemIcon div {
   display: inline-flex;
   vertical-align: middle;
 }
 .tray-item {
   cursor: pointer;
 }