export const datatype_test_1 = `Executing: AllLiteralTypes
String inPort:
abc
Integer inPort:
123
Float inPort:
12.3
Boolean inPort:
True
List inPort:
['a', 'b', 'c']
Tuple inPort:
('a', 'b', 'c')
Dict inPort:
{'a': 'apple', 'b': 'banana', 'c': 2022}
Secret inPort:
554
Chat inPort:
[{'role': 'system', 'content': 'abc'}, {'role': 'user', 'content': 'def'}]`

export const datatype_test_2 = `Executing: AllLiteralTypes
String inPort:
Updated String
Integer inPort:
4560.0
Float inPort:
45.6
Boolean inPort:
False
List inPort:
['d', 'e', 'f']
Tuple inPort:
('g', 'h', 'i')
Dict inPort:
{'x': 'xenon', 'y': 'yellow', 'z': 2023}
Secret inPort:
def
Chat inPort:
[{'role': 'user', 'content': 'updated user message'}, {'role': 'assistant', 'content': 'new assistant message'}]`